@@ -10,9 +10,6 @@ use serde::{Deserialize, Serialize};
 
 use super::asset_info::{AssetInfo, AssetInfoBase};
 
-#[cfg(feature = "terra")]
-static DECIMAL_FRACTION: Uint128 = Uint128::new(1_000_000_000_000_000_000u128);
-
 /// Represents a fungible asset with a known amount
 ///
 /// Each asset instance contains two values: [`info`], which specifies the asset's type (CW20 or
@@ -98,22 +95,25 @@ impl From<Asset> for AssetUnchecked {
 
 impl AssetUnchecked {
     /// Validate data contained in an _unchecked_ **asset** instnace, return a new _checked_
-    /// **asset** instance
+    /// **asset** instance:
+    /// * For CW20 tokens, assert the contract address is valid
+    /// * For SDK coins, assert that the denom is included in a given whitelist; skip if the 
+    ///   whitelist is not provided
     ///
     /// ```rust
     /// use cosmwasm_std::{Addr, Api};
     /// use cw_asset::{Asset, AssetUnchecked};
     ///
     /// fn validate_asset(api: &dyn Api, asset_unchecked: &AssetUnchecked) {
-    ///     match asset_unchecked.check(api) {
+    ///     match asset_unchecked.check(api, Some(&["uatom", "uluna"])) {
     ///         Ok(asset) => println!("asset is valid: {}", asset.to_string()),
     ///         Err(err) => println!("asset is invalid! reason: {}", err)
     ///     }
     /// }
     /// ```
-    pub fn check(&self, api: &dyn Api) -> StdResult<Asset> {
+    pub fn check(&self, api: &dyn Api, optional_whitelist: Option<&[&str]>) -> StdResult<Asset> {
         Ok(Asset {
-            info: self.info.check(api)?,
+            info: self.info.check(api, optional_whitelist)?,
             amount: self.amount,
         })
     }
@@ -307,6 +307,7 @@ impl std::cmp::PartialEq<astroport::asset::Asset> for Asset {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use crate::AssetInfoUnchecked;
     use cosmwasm_std::testing::MockApi;
 
     #[derive(Serialize)]
@@ -367,13 +368,22 @@ mod tests {
     }
 
     #[test]
-    fn casting() {
+    fn checking() {
         let api = MockApi::default();
 
-        let checked = Asset::cw20(Addr::unchecked("mock_token"), 123456u128);
+        let checked = Asset::cw20(Addr::unchecked("mock_token"), 12345u128);
         let unchecked: AssetUnchecked = checked.clone().into();
+        assert_eq!(unchecked.check(&api, None).unwrap(), checked);
 
-        assert_eq!(unchecked.check(&api).unwrap(), checked);
+        let checked = Asset::native("uusd", 12345u128);
+        let unchecked: AssetUnchecked = checked.clone().into();
+        assert_eq!(unchecked.check(&api, Some(&["uusd", "uluna", "uosmo"])).unwrap(), checked);
+
+        let unchecked = AssetUnchecked::new(AssetInfoUnchecked::native("uatom"), 12345u128);
+        assert_eq!(
+            unchecked.check(&api, Some(&["uusd", "uluna", "uosmo"])),
+            Err(StdError::generic_err("invalid denom uatom; must be uusd|uluna|uosmo")),
+        );
     }
 
     #[test]
